"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndyVdrAnonCredsRegistry = void 0;
const core_1 = require("@aries-framework/core");
const indy_vdr_shared_1 = require("@hyperledger/indy-vdr-shared");
const pool_1 = require("../pool");
const identifiers_1 = require("./utils/identifiers");
const transform_1 = require("./utils/transform");
class IndyVdrAnonCredsRegistry {
    constructor() {
        this.supportedIdentifier = identifiers_1.indyVdrAnonCredsRegistryIdentifierRegex;
    }
    async getSchema(agentContext, schemaId) {
        try {
            const indyVdrPoolService = agentContext.dependencyManager.resolve(pool_1.IndyVdrPoolService);
            const did = (0, identifiers_1.didFromSchemaId)(schemaId);
            const pool = await indyVdrPoolService.getPoolForDid(agentContext, did);
            agentContext.config.logger.debug(`Getting schema '${schemaId}' from ledger '${pool.indyNamespace}'`);
            const request = new indy_vdr_shared_1.GetSchemaRequest({ submitterDid: did, schemaId });
            agentContext.config.logger.trace(`Submitting get schema request for schema '${schemaId}' to ledger '${pool.indyNamespace}'`);
            const response = await pool.submitReadRequest(request);
            agentContext.config.logger.trace(`Got un-parsed schema '${schemaId}' from ledger '${pool.indyNamespace}'`, {
                response,
            });
            const issuerId = (0, identifiers_1.didFromSchemaId)(schemaId);
            if ('attr_names' in response.result.data) {
                return {
                    schema: {
                        attrNames: response.result.data.attr_names,
                        name: response.result.data.name,
                        version: response.result.data.version,
                        issuerId,
                    },
                    schemaId: schemaId,
                    resolutionMetadata: {},
                    schemaMetadata: {
                        didIndyNamespace: pool.indyNamespace,
                        // NOTE: the seqNo is required by the indy-sdk even though not present in AnonCreds v1.
                        // For this reason we return it in the metadata.
                        indyLedgerSeqNo: response.result.seqNo,
                    },
                };
            }
            agentContext.config.logger.error(`Error retrieving schema '${schemaId}'`);
            return {
                schemaId,
                resolutionMetadata: {
                    error: 'notFound',
                    message: `unable to find schema with id ${schemaId}`,
                },
                schemaMetadata: {},
            };
        }
        catch (error) {
            agentContext.config.logger.error(`Error retrieving schema '${schemaId}'`, {
                error,
                schemaId,
            });
            return {
                schemaId,
                resolutionMetadata: {
                    error: 'notFound',
                },
                schemaMetadata: {},
            };
        }
    }
    async registerSchema(agentContext, options) {
        if (!options.options.didIndyNamespace) {
            return {
                schemaMetadata: {},
                registrationMetadata: {},
                schemaState: {
                    reason: 'no didIndyNamespace defined in the options. didIndyNamespace is required when using the Indy VDR',
                    schema: options.schema,
                    state: 'failed',
                },
            };
        }
        try {
            const indyVdrPoolService = agentContext.dependencyManager.resolve(pool_1.IndyVdrPoolService);
            const schemaRequest = new indy_vdr_shared_1.SchemaRequest({
                submitterDid: options.schema.issuerId,
                schema: {
                    id: (0, identifiers_1.getLegacySchemaId)(options.schema.issuerId, options.schema.name, options.schema.version),
                    name: options.schema.name,
                    ver: '1.0',
                    version: options.schema.version,
                    attrNames: options.schema.attrNames,
                },
            });
            const pool = indyVdrPoolService.getPoolForNamespace(options.options.didIndyNamespace);
            // FIXME: we should store the didDocument in the DidRecord so we don't have to fetch our own did
            // from the ledger to know which key is associated with the did
            const didsApi = agentContext.dependencyManager.resolve(core_1.DidsApi);
            const didResult = await didsApi.resolve(`did:sov:${options.schema.issuerId}`);
            if (!didResult.didDocument) {
                return {
                    schemaMetadata: {},
                    registrationMetadata: {},
                    schemaState: {
                        schema: options.schema,
                        state: 'failed',
                        reason: `didNotFound: unable to resolve did did:sov:${options.schema.issuerId}: ${didResult.didResolutionMetadata.message}`,
                    },
                };
            }
            const verificationMethod = didResult.didDocument.dereferenceKey(`did:sov:${options.schema.issuerId}#key-1`);
            const { getKeyFromVerificationMethod } = (0, core_1.getKeyDidMappingByVerificationMethod)(verificationMethod);
            const key = getKeyFromVerificationMethod(verificationMethod);
            const response = await pool.submitWriteRequest(agentContext, schemaRequest, key);
            return {
                schemaState: {
                    state: 'finished',
                    schema: {
                        attrNames: options.schema.attrNames,
                        issuerId: options.schema.issuerId,
                        name: options.schema.name,
                        version: options.schema.version,
                    },
                    schemaId: (0, identifiers_1.getLegacySchemaId)(options.schema.issuerId, options.schema.name, options.schema.version),
                },
                registrationMetadata: {},
                schemaMetadata: {
                    // NOTE: the seqNo is required by the indy-sdk even though not present in AnonCreds v1.
                    // For this reason we return it in the metadata.
                    indyLedgerSeqNo: response.result.txnMetadata.seqNo,
                    didIndyNamespace: pool.indyNamespace,
                },
            };
        }
        catch (error) {
            agentContext.config.logger.error(`Error registering schema for did '${options.schema.issuerId}'`, {
                error,
                did: options.schema.issuerId,
                schema: options.schema,
            });
            return {
                schemaMetadata: {},
                registrationMetadata: {},
                schemaState: {
                    state: 'failed',
                    schema: options.schema,
                    reason: `unknownError: ${error.message}`,
                },
            };
        }
    }
    async getCredentialDefinition(agentContext, credentialDefinitionId) {
        try {
            const indyVdrPoolService = agentContext.dependencyManager.resolve(pool_1.IndyVdrPoolService);
            const did = (0, identifiers_1.didFromCredentialDefinitionId)(credentialDefinitionId);
            const pool = await indyVdrPoolService.getPoolForDid(agentContext, did);
            agentContext.config.logger.debug(`Getting credential definition '${credentialDefinitionId}' from ledger '${pool.indyNamespace}'`);
            const request = new indy_vdr_shared_1.GetCredentialDefinitionRequest({
                submitterDid: did,
                credentialDefinitionId,
            });
            agentContext.config.logger.trace(`Submitting get credential definition request for credential definition '${credentialDefinitionId}' to ledger '${pool.indyNamespace}'`);
            const response = await pool.submitReadRequest(request);
            const schema = await this.fetchIndySchemaWithSeqNo(agentContext, response.result.ref, did);
            if (response.result.data && schema) {
                return {
                    credentialDefinitionId: credentialDefinitionId,
                    credentialDefinition: {
                        issuerId: (0, identifiers_1.didFromCredentialDefinitionId)(credentialDefinitionId),
                        schemaId: schema.schema.schemaId,
                        tag: response.result.tag,
                        type: 'CL',
                        value: response.result.data,
                    },
                    credentialDefinitionMetadata: {
                        didIndyNamespace: pool.indyNamespace,
                    },
                    resolutionMetadata: {},
                };
            }
            agentContext.config.logger.error(`Error retrieving credential definition '${credentialDefinitionId}'`);
            return {
                credentialDefinitionId,
                credentialDefinitionMetadata: {},
                resolutionMetadata: {
                    error: 'notFound',
                    message: `unable to resolve credential definition with id ${credentialDefinitionId}`,
                },
            };
        }
        catch (error) {
            agentContext.config.logger.error(`Error retrieving credential definition '${credentialDefinitionId}'`, {
                error,
                credentialDefinitionId,
            });
            return {
                credentialDefinitionId,
                credentialDefinitionMetadata: {},
                resolutionMetadata: {
                    error: 'notFound',
                    message: `unable to resolve credential definition: ${error.message}`,
                },
            };
        }
    }
    async registerCredentialDefinition(agentContext, options) {
        // Make sure didIndyNamespace is passed
        if (!options.options.didIndyNamespace) {
            return {
                credentialDefinitionMetadata: {},
                registrationMetadata: {},
                credentialDefinitionState: {
                    reason: 'no didIndyNamespace defined in the options. didIndyNamespace is required when using the Indy SDK',
                    credentialDefinition: options.credentialDefinition,
                    state: 'failed',
                },
            };
        }
        try {
            const indyVdrPoolService = agentContext.dependencyManager.resolve(pool_1.IndyVdrPoolService);
            const pool = indyVdrPoolService.getPoolForNamespace(options.options.didIndyNamespace);
            const { schema, schemaMetadata, resolutionMetadata } = await this.getSchema(agentContext, options.credentialDefinition.schemaId);
            if (!schema || !schemaMetadata.indyLedgerSeqNo || typeof schemaMetadata.indyLedgerSeqNo !== 'number') {
                return {
                    registrationMetadata: {},
                    credentialDefinitionMetadata: {
                        didIndyNamespace: pool.indyNamespace,
                    },
                    credentialDefinitionState: {
                        credentialDefinition: options.credentialDefinition,
                        state: 'failed',
                        reason: `error resolving schema with id ${options.credentialDefinition.schemaId}: ${resolutionMetadata.error} ${resolutionMetadata.message}`,
                    },
                };
            }
            const credentialDefinitionId = (0, identifiers_1.getLegacyCredentialDefinitionId)(options.credentialDefinition.issuerId, schemaMetadata.indyLedgerSeqNo, options.credentialDefinition.tag);
            const credentialDefinitionRequest = new indy_vdr_shared_1.CredentialDefinitionRequest({
                submitterDid: options.credentialDefinition.issuerId,
                credentialDefinition: {
                    ver: '1.0',
                    id: credentialDefinitionId,
                    schemaId: `${schemaMetadata.indyLedgerSeqNo}`,
                    type: 'CL',
                    tag: options.credentialDefinition.tag,
                    value: options.credentialDefinition.value,
                },
            });
            // FIXME: we should store the didDocument in the DidRecord so we don't have to fetch our own did
            // from the ledger to know which key is associated with the did
            const didsApi = agentContext.dependencyManager.resolve(core_1.DidsApi);
            const didResult = await didsApi.resolve(`did:sov:${options.credentialDefinition.issuerId}`);
            if (!didResult.didDocument) {
                return {
                    credentialDefinitionMetadata: {},
                    registrationMetadata: {},
                    credentialDefinitionState: {
                        credentialDefinition: options.credentialDefinition,
                        state: 'failed',
                        reason: `didNotFound: unable to resolve did did:sov:${options.credentialDefinition.issuerId}: ${didResult.didResolutionMetadata.message}`,
                    },
                };
            }
            const verificationMethod = didResult.didDocument.dereferenceKey(`did:sov:${options.credentialDefinition.issuerId}#key-1`);
            const { getKeyFromVerificationMethod } = (0, core_1.getKeyDidMappingByVerificationMethod)(verificationMethod);
            const key = getKeyFromVerificationMethod(verificationMethod);
            const response = await pool.submitWriteRequest(agentContext, credentialDefinitionRequest, key);
            agentContext.config.logger.debug(`Registered credential definition '${credentialDefinitionId}' on ledger '${pool.indyNamespace}'`, {
                response,
                credentialDefinition: options.credentialDefinition,
            });
            return {
                credentialDefinitionMetadata: {
                    didIndyNamespace: pool.indyNamespace,
                },
                credentialDefinitionState: {
                    credentialDefinition: options.credentialDefinition,
                    credentialDefinitionId,
                    state: 'finished',
                },
                registrationMetadata: {},
            };
        }
        catch (error) {
            agentContext.config.logger.error(`Error registering credential definition for schema '${options.credentialDefinition.schemaId}'`, {
                error,
                did: options.credentialDefinition.issuerId,
                credentialDefinition: options.credentialDefinition,
            });
            return {
                credentialDefinitionMetadata: {},
                registrationMetadata: {},
                credentialDefinitionState: {
                    credentialDefinition: options.credentialDefinition,
                    state: 'failed',
                    reason: `unknownError: ${error.message}`,
                },
            };
        }
    }
    async getRevocationRegistryDefinition(agentContext, revocationRegistryDefinitionId) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        try {
            const indySdkPoolService = agentContext.dependencyManager.resolve(pool_1.IndyVdrPoolService);
            const did = (0, identifiers_1.didFromRevocationRegistryDefinitionId)(revocationRegistryDefinitionId);
            const pool = await indySdkPoolService.getPoolForDid(agentContext, did);
            agentContext.config.logger.debug(`Using ledger '${pool.indyNamespace}' to retrieve revocation registry definition '${revocationRegistryDefinitionId}'`);
            const request = new indy_vdr_shared_1.GetRevocationRegistryDefinitionRequest({
                submitterDid: did,
                revocationRegistryId: revocationRegistryDefinitionId,
            });
            agentContext.config.logger.trace(`Submitting get revocation registry definition request for revocation registry definition '${revocationRegistryDefinitionId}' to ledger`);
            const response = await pool.submitReadRequest(request);
            if (!response.result.data) {
                agentContext.config.logger.error(`Error retrieving revocation registry definition '${revocationRegistryDefinitionId}' from ledger`, {
                    revocationRegistryDefinitionId,
                });
                return {
                    resolutionMetadata: {
                        error: 'notFound',
                        message: `unable to resolve revocation registry definition`,
                    },
                    revocationRegistryDefinitionId,
                    revocationRegistryDefinitionMetadata: {},
                };
            }
            const revocationRegistryDefinition = {
                issuerId: did,
                revocDefType: (_a = response.result.data) === null || _a === void 0 ? void 0 : _a.revocDefType,
                value: {
                    maxCredNum: (_b = response.result.data) === null || _b === void 0 ? void 0 : _b.value.maxCredNum,
                    tailsHash: (_c = response.result.data) === null || _c === void 0 ? void 0 : _c.value.tailsHash,
                    tailsLocation: (_d = response.result.data) === null || _d === void 0 ? void 0 : _d.value.tailsLocation,
                    publicKeys: {
                        accumKey: {
                            z: (_e = response.result.data) === null || _e === void 0 ? void 0 : _e.value.publicKeys.accumKey.z,
                        },
                    },
                },
                tag: (_f = response.result.data) === null || _f === void 0 ? void 0 : _f.tag,
                credDefId: (_g = response.result.data) === null || _g === void 0 ? void 0 : _g.credDefId,
            };
            return {
                revocationRegistryDefinitionId,
                revocationRegistryDefinition,
                revocationRegistryDefinitionMetadata: {
                    issuanceType: (_h = response.result.data) === null || _h === void 0 ? void 0 : _h.value.issuanceType,
                    didIndyNamespace: pool.indyNamespace,
                },
                resolutionMetadata: {},
            };
        }
        catch (error) {
            agentContext.config.logger.error(`Error retrieving revocation registry definition '${revocationRegistryDefinitionId}' from ledger`, {
                error,
                revocationRegistryDefinitionId,
            });
            return {
                resolutionMetadata: {
                    error: 'notFound',
                    message: `unable to resolve revocation registry definition: ${error.message}`,
                },
                revocationRegistryDefinitionId,
                revocationRegistryDefinitionMetadata: {},
            };
        }
    }
    async getRevocationStatusList(agentContext, revocationRegistryId, timestamp) {
        var _a, _b, _c;
        try {
            const indySdkPoolService = agentContext.dependencyManager.resolve(pool_1.IndyVdrPoolService);
            const did = (0, identifiers_1.didFromRevocationRegistryDefinitionId)(revocationRegistryId);
            const pool = await indySdkPoolService.getPoolForDid(agentContext, did);
            agentContext.config.logger.debug(`Using ledger '${pool.indyNamespace}' to retrieve revocation registry deltas with revocation registry definition id '${revocationRegistryId}' until ${timestamp}`);
            const request = new indy_vdr_shared_1.GetRevocationRegistryDeltaRequest({
                submitterDid: did,
                revocationRegistryId,
                toTs: timestamp,
            });
            agentContext.config.logger.trace(`Submitting get revocation registry delta request for revocation registry '${revocationRegistryId}' to ledger`);
            const response = await pool.submitReadRequest(request);
            agentContext.config.logger.debug(`Got revocation registry deltas '${revocationRegistryId}' until timestamp ${timestamp} from ledger`);
            const { revocationRegistryDefinition, resolutionMetadata, revocationRegistryDefinitionMetadata } = await this.getRevocationRegistryDefinition(agentContext, revocationRegistryId);
            if (!revocationRegistryDefinition ||
                !revocationRegistryDefinitionMetadata.issuanceType ||
                typeof revocationRegistryDefinitionMetadata.issuanceType !== 'string') {
                return {
                    resolutionMetadata: {
                        error: `error resolving revocation registry definition with id ${revocationRegistryId}: ${resolutionMetadata.error} ${resolutionMetadata.message}`,
                    },
                    revocationStatusListMetadata: {
                        didIndyNamespace: pool.indyNamespace,
                    },
                };
            }
            const isIssuanceByDefault = revocationRegistryDefinitionMetadata.issuanceType === 'ISSUANCE_BY_DEFAULT';
            if (!response.result.data) {
                return {
                    resolutionMetadata: {
                        error: 'notFound',
                        message: `Error retrieving revocation registry delta '${revocationRegistryId}' from ledger, potentially revocation interval ends before revocation registry creation`,
                    },
                    revocationStatusListMetadata: {},
                };
            }
            const revocationRegistryDelta = {
                accum: (_a = response.result.data) === null || _a === void 0 ? void 0 : _a.value.accum_to.value.accum,
                issued: (_b = response.result.data) === null || _b === void 0 ? void 0 : _b.value.issued,
                revoked: (_c = response.result.data) === null || _c === void 0 ? void 0 : _c.value.revoked,
            };
            return {
                resolutionMetadata: {},
                revocationStatusList: (0, transform_1.anonCredsRevocationStatusListFromIndyVdr)(revocationRegistryId, revocationRegistryDefinition, revocationRegistryDelta, response.result.data.value.accum_to.txnTime, isIssuanceByDefault),
                revocationStatusListMetadata: {
                    didIndyNamespace: pool.indyNamespace,
                },
            };
        }
        catch (error) {
            agentContext.config.logger.error(`Error retrieving revocation registry delta '${revocationRegistryId}' from ledger, potentially revocation interval ends before revocation registry creation?"`, {
                error,
                revocationRegistryId: revocationRegistryId,
            });
            return {
                resolutionMetadata: {
                    error: 'notFound',
                    message: `Error retrieving revocation registry delta '${revocationRegistryId}' from ledger, potentially revocation interval ends before revocation registry creation: ${error.message}`,
                },
                revocationStatusListMetadata: {},
            };
        }
    }
    async fetchIndySchemaWithSeqNo(agentContext, seqNo, did) {
        var _a, _b;
        const indyVdrPoolService = agentContext.dependencyManager.resolve(pool_1.IndyVdrPoolService);
        const pool = await indyVdrPoolService.getPoolForDid(agentContext, did);
        agentContext.config.logger.debug(`Getting transaction with seqNo '${seqNo}' from ledger '${pool.indyNamespace}'`);
        // ledgerType 1 is domain ledger
        const request = new indy_vdr_shared_1.GetTransactionRequest({ ledgerType: 1, seqNo });
        agentContext.config.logger.trace(`Submitting get transaction request to ledger '${pool.indyNamespace}'`);
        const response = await pool.submitReadRequest(request);
        if (((_a = response.result.data) === null || _a === void 0 ? void 0 : _a.txn.type) !== '101') {
            agentContext.config.logger.error(`Could not get schema from ledger for seq no ${seqNo}'`);
            return null;
        }
        const schema = (_b = response.result.data) === null || _b === void 0 ? void 0 : _b.txn.data;
        const schemaId = (0, identifiers_1.getLegacySchemaId)(did, schema.data.name, schema.data.version);
        return {
            schema: {
                schemaId,
                attr_name: schema.data.attr_names,
                name: schema.data.name,
                version: schema.data.version,
                issuerId: did,
                seqNo,
            },
            indyNamespace: pool.indyNamespace,
        };
    }
}
exports.IndyVdrAnonCredsRegistry = IndyVdrAnonCredsRegistry;
//# sourceMappingURL=IndyVdrAnonCredsRegistry.js.map